# TPK4186 - Advanced Tools for Performance Engineering
# BoxMullerMethod.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

import random
import math
import numpy as np
import matplotlib.pyplot as plt

def BoXMullerDraw():
  U = random.random()
  V = random.random()
  X = math.sqrt(-2*math.log(U)) * math.cos(2*math.pi*V)
  Y = math.sqrt(-2*math.log(U)) * math.sin(2*math.pi*V)
  return (X, Y)

def GenerateSamples(size):
  Xs = np.empty(size)
  Ys = np.empty(size)
  for i in range(0, size):
    (X, Y) = BoXMullerDraw()
    Xs[i] = X
    Ys[i] = Y
  return (Xs, Ys)
  
size = 1000000
(Xs, Ys) = GenerateSamples(size)
Xmean = Xs.mean()
Ymean = Ys.mean()
Xstd = Xs.std()
Ystd = Ys.std()
print(str(Xmean) + "\t" + str(Xstd))
print(str(Ymean) + "\t" + str(Ystd))

bins = [-3, -2, -1, 0, 1, 2, 3, 4]
HXs, bs = np.histogram(Xs, bins, density=True)
HYs, bs = np.histogram(Ys, bins, density=True)
print(HXs) 
print(HYs)

numberOfBins = 20
_size, _bins, _patches = plt.hist(Xs, numberOfBins, facecolor='blue', alpha=0.5)
plt.show()
_size, _bins, _patches = plt.hist(Ys, numberOfBins, facecolor='green', alpha=0.5)
plt.show()

bins = [0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0]
QXs = np.quantile(Xs, bins)
QYs = np.quantile(Ys, bins)
print(QXs) 
print(QYs)
plt.plot(QXs)
plt.show()
