# TPK4186 - Advanced Tools for Performance Engineering
# HeightDistribution.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

import sys
import math
import numpy as np
import matplotlib.pyplot as plt

# Q1
def LoadSampleFromCSVFile(fileName, delimiter):
  try:
    sample = np.genfromtxt(fileName, delimiter=delimiter)
  except:
    sys.stderr.write("Unable to open file " + fileName + "\n")
    return None
  return sample

sample = LoadSampleFromCSVFile("Heights.csv", "\t")
genders = sample[:, 2]
numberOfMen = np.count_nonzero(genders==1)
numberOfWomen = np.count_nonzero(genders==2)
sizeSample = numberOfMen + numberOfWomen
print("Men\tWomen\tTotal")
print("{0:d}\t{1:d}\t{2:d}".format(numberOfMen, numberOfWomen, sizeSample))
print("{0:.2f}\t{1:.2f}\t1.0".format(numberOfMen/sizeSample, numberOfWomen/sizeSample))

# Q2
menSample = sample[:][sample[:,2]==1]
womenSample = sample[:][sample[:,2]==2]
minMen = np.amin(menSample[:,3])
maxMen = np.amax(menSample[:,3])
meanMen = np.mean(menSample[:,3])
minWomen = np.amin(womenSample[:,3])
maxWomen = np.amax(womenSample[:,3])
meanWomen = np.mean(womenSample[:,3])
print("    \tMen\tWomen")
print("min \t{0:.1f}\t{1:.1f}".format(minMen, minWomen))
print("max \t{0:.1f}\t{1:.1f}".format(maxMen, maxWomen))
print("mean\t{0:.1f}\t{1:.1f}".format(meanMen, meanWomen))

# Q3
n, bins, patches = plt.hist(menSample[:,3], bins=np.arange(minMen, maxMen+1)-0.5, density=100, facecolor='blue')
plt.xlabel('Heights')
plt.ylabel('Percentage')
plt.title('Men height distribution')
plt.show()
n, bins, patches = plt.hist(womenSample[:,3], bins=np.arange(minWomen, maxWomen+1)-0.5, density=100, facecolor='green')
plt.xlabel('Heights')
plt.ylabel('Percentage')
plt.title('Women height distribution')
plt.show()

# Q4
menSample1 = menSample[:][menSample[:,1]>=18]
menSample1 = menSample1[:][menSample1[:,1]<=29]
menSample2 = menSample[:][menSample[:,1]>=30]
menSample2 = menSample2[:][menSample2[:,1]<=54]
menSample3 = menSample[:][menSample[:,1]>=55]
menSample3 = menSample3[:][menSample3[:,1]<=74]
# menSample3 = menSample[:][menSample[:,1]>=55 and menSample[:,1]<=74]
n, bins, patches = plt.hist(menSample1[:,3], bins=np.arange(minMen, maxMen+1)-0.5, density=100, color='blue', histtype='step', cumulative=True)
n, bins, patches = plt.hist(menSample2[:,3], bins=np.arange(minMen, maxMen+1)-0.5, density=100, color='green', histtype='step', cumulative=True)
n, bins, patches = plt.hist(menSample3[:,3], bins=np.arange(minMen, maxMen+1)-0.5, density=100, color='red', histtype='step', cumulative=True)
plt.xlabel('Heights')
plt.ylabel('Percentage')
plt.title('Men height distribution')
plt.show()

womenSample1 = womenSample[:][womenSample[:,1]>=18]
womenSample1 = womenSample1[:][womenSample1[:,1]<=29]
womenSample2 = womenSample[:][womenSample[:,1]>=30]
womenSample2 = womenSample2[:][womenSample2[:,1]<=54]
womenSample3 = womenSample[:][womenSample[:,1]>=55]
womenSample3 = womenSample3[:][womenSample3[:,1]<=74]
# womenSample3 = womenSample[:][womenSample[:,1]>=55 and womenSample[:,1]<=74]
n, bins, patches = plt.hist(womenSample1[:,3], bins=np.arange(minWomen, maxWomen+1)-0.5, density=100, color='blue', histtype='step', cumulative=True)
n, bins, patches = plt.hist(womenSample2[:,3], bins=np.arange(minWomen, maxWomen+1)-0.5, density=100, color='green', histtype='step', cumulative=True)
n, bins, patches = plt.hist(womenSample3[:,3], bins=np.arange(minWomen, maxWomen+1)-0.5, density=100, color='red', histtype='step', cumulative=True)
plt.xlabel('Heights')
plt.ylabel('Percentage')
plt.title('Women height distribution')
plt.show()

