# TPK4186 - Advanced Tools for Performance Engineering
# KOutOfN.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

import random
import math

# Q1
def CalculateProbability(ps, k):
  n = len(ps)
  Qs = [0.0 for i in range(0, k+1)]
  Qs[0] = 1.0
  for i in range(0, n):
    Qs[k] = ps[i]*Qs[k-1] + Qs[k]
    for j in range(k-1, 0, -1):
      Qs[j] = ps[i]*Qs[j-1] + (1-ps[i])*Qs[j]
    Qs[0] = (1-ps[i])*Qs[0]
  return Qs[k]

# Q2
k = 4
n = 10
ps = [0.0 for i in range(0, n)]
p = 0.1
while p>=0.01:
  for i in range(0, n):
    ps[i] = p
  Q = CalculateProbability(ps, k)
  print("{0:.2f}\t{1:1.3e}".format(p, Q))
  p -= 0.01

# Q3
def MonteCarloSimulation(ps, k, numberOfTrials):
  n = len(ps)
  N0 = 0
  N1 = 0
  N = numberOfTrials
  for trial in range(0, N):
    j = 0
    for i in range(0, n):
      z = random.random()
      if z<ps[i]:
        j += 1
    if j>=k:
      N1 += 1
    else:
      N0 += 1
  mean = N1/N
  EF90 = (1.64/N) * math.sqrt(N1*N0 / N)
  EF95 = (1.96/N) * math.sqrt(N1*N0 / N)
  EF99 = (2.68/N) * math.sqrt(N1*N0 / N)
  print("mean\t{0:1.3e}".format(mean))
  print("CR90\t{0:1.3e}\t{1:1.3e}".format(mean-EF90, mean+EF90))
  print("CR95\t{0:1.3e}\t{1:1.3e}".format(mean-EF95, mean+EF95))
  print("CR99\t{0:1.3e}\t{1:1.3e}".format(mean-EF99, mean+EF99))

# Q4
k = 4
n = 10
p = 0.01
ps = [p for i in range(0, n)]
N = 10000000
print("\np={0:1.2f}, Q={1:1.3e}, N={2:d}".format(p, CalculateProbability(ps, k), N))
MonteCarloSimulation(ps, k, N)

