# TPK4186 - Advanced Tools for Performance Engineering
# PatientSurvival.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

import sys
import numpy as np
import matplotlib.pyplot as plt

def LoadSampleFromCSVFile(fileName, delimiter):
  try:
    sample = np.genfromtxt(fileName, delimiter=delimiter)
  except:
    sys.stderr.write("Unable to open file " + fileName + "\n")
    return None
  return sample

deaths = LoadSampleFromCSVFile("Deaths.csv", "\t")
deaths[0, :] = 0
group1Size = 23
group2Size = 23
dates = deaths[:, 0]
totalDeaths = np.cumsum(deaths[:, 1])
group2Deaths = np.cumsum(deaths[:, 2])
group1SurvivalProbability = (np.full(totalDeaths.size, group1Size) - totalDeaths + group2Deaths) / group1Size
group2SurvivalProbability = (np.full(totalDeaths.size, group2Size) - group2Deaths) / group2Size

plt.plot(dates, group1SurvivalProbability, "b-", label="group 1")
plt.plot(dates, group2SurvivalProbability, "g-", label="group 2")
plt.xlabel('Dates (in days)')
plt.ylabel('Survival probability')
plt.title('Survival curves of patients with cancer of type A')
plt.legend()
