# TPK4186 - Advanced Tools for Performance Engineering
# WealthDistributionOnTheFly.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

import sys
import math

def AnalyzeOnTheFlySampleFromCSVFile(fileName, delimiter):
  try:
    file = open(fileName, "r")
  except:
    sys.stderr.write("Unable to open file " + fileName + "\n")
    return
  size = 0 
  sum = 0.0
  sum2 = 0.0
  for line in file:
   line = line.rstrip()
   if line!="":
      tokens = line.split(delimiter)
      size += 1
      wealth = int(tokens[1])
      sum += wealth
      sum2 += wealth**2
  mean = sum/size
  stddev = math.sqrt(sum2/size - mean**2)
  errfac90 = 1.64*stddev/math.sqrt(size)
  errfac95 = 1.96*stddev/math.sqrt(size)
  errfac99 = 2.58*stddev/math.sqrt(size)
  print("mean   \t{0:.0f}".format(mean))
  print("stddev \t{0:.0f}".format(stddev))
  print("90% rng\t{0:.0f}\t{1:.0f}".format(mean-errfac90, mean+errfac90))
  print("95% rng\t{0:.0f}\t{1:.0f}".format(mean-errfac95, mean+errfac95))
  print("99% rng\t{0:.0f}\t{1:.0f}".format(mean-errfac99, mean+errfac99))


AnalyzeOnTheFlySampleFromCSVFile("Norway2018.csv", "\t")


def CreateAPrioriBinsOfHistogramOfSample(minimumValue, maximumValue, numberOfBins):
  step = (maximumValue-minimumValue)/numberOfBins
  bins = []
  value = minimumValue+step
  while value<=maximumValue:
    bins.append(value)
    value += step
  bins[numberOfBins-1] += 1
  return bins
 
def ExtractHistogramOfSampleFromCSVFile(fileName, delimiter, bins):
  try:
    file = open(fileName, "r")
  except:
    sys.stderr.write("Unable to open file " + fileName + "\n")
    return
  numberOfBins = len(bins)
  histogram = [0 for bin in range(0, numberOfBins)]
  for line in file:
    line = line.rstrip()
    if line=="":
      continue
    tokens = line.split(delimiter)
    value = int(tokens[1])
    for binIndex in range(0, numberOfBins):
      if value<=bins[binIndex]:
        histogram[binIndex] += 1
        break
  return histogram

bins = CreateAPrioriBinsOfHistogramOfSample(-2.0e6, 500e6, 20)
histogram = ExtractHistogramOfSampleFromCSVFile("Norway2018.csv", "\t", bins)
print(bins)
print(histogram)
   